# Onboarding and instantiation

```bash
osm nfpkg-create k8s_base_vnf.tar.gz
osm nfpkg-create k8s_juju_vnf.tar.gz
osm nspkg-create k8s_base_ns.tar.gz
osm ns-create --ns_name k8s --nsd_name k8s_base --vim_account ost9-canonical-fortville --config '{vld: [ {name: mgmtnet, vim-network-name: mgmt} ] }'
```

This deploys a NS (k8s\_base\_ns) with one VNF that runs Juju (k8s\_juju\_vnf) and 4 VNFs (k8s\_base\_vnf) that run the K8s cluster (1 master, 3 workers).

VMs will be called: 

- JUJU
- NODE1
- NODE2
- NODE3
- NODE4

# Install and bootstrap of Juju in the VM where juju runs (JUJU)

## Install juju

```bash
sudo snap install juju --classic --channel=2.6/stable
ssh-keygen
```

## Add the cloud: manual provider type

```bash
# Copy the ssh key locally
ssh-copy-id ubuntu@<IP_JUJU>
juju add-cloud
    manual
    k8s
    ubuntu@<IP_JUJU>
```

## Create the controller

```bash
ssh-copy-id -i /home/ubuntu/.local/share/juju/ssh/juju_id_rsa ubuntu@<IP_JUJU>
#The same as: cat /home/ubuntu/.local/share/juju/ssh/juju_id_rsa.pub >> .ssh/authorized_keys
juju bootstrap k8s jujuk8s
```

This creates a controller locally in the machine

Juju status can be checked with:

```
juju status
```

# Add the VMs to juju (manual provider mode)

First from JUJU you need to copy the controller SSH key to all nodes of the cluster (master and workers)

```bash
#cat /home/ubuntu/.local/share/juju/ssh/juju_id_rsa.pub
#vi .ssh/authorized_keys
ssh-copy-id -i /home/ubuntu/.local/share/juju/ssh/juju_id_rsa ubuntu@<IP_NODE1>
ssh-copy-id -i /home/ubuntu/.local/share/juju/ssh/juju_id_rsa ubuntu@<IP_NODE2>
ssh-copy-id -i /home/ubuntu/.local/share/juju/ssh/juju_id_rsa ubuntu@<IP_NODE3>
ssh-copy-id -i /home/ubuntu/.local/share/juju/ssh/juju_id_rsa ubuntu@<IP_NODE4>
```

After that add the machines to juju controller mannually in JUJU: 

```bash
juju add-machine ssh:ubuntu@<IP_NODE1> --debug
juju add-machine ssh:ubuntu@<IP_NODE2> --debug
juju add-machine ssh:ubuntu@<IP_NODE3> --debug
juju add-machine ssh:ubuntu@<IP_NODE4> --debug
```

Next we look for the indentifiers of the machines and take notes of it.

```bash
juju machines
```

If they are the first machines we add, then the ids will be 0, 1, 2, 3.

# Deploy the K8s cluster with the bundle

The original bundle cannot be used as it is because it assumes that it is going to be deployed in a cloud without specifying the machines.

```bash
#juju deploy charmed-kubernetes
#wget https://api.jujucharms.com/charmstore/v5/charmed-kubernetes/archive/bundle.yaml
#vi bundle.yaml
```

The bundle has been edited to specify which application runs in which machine.

We copy the bundle from osm-descriptors repo to JUJU:

```bash
scp osm-descriptors/tid/k8s/k8s_base_ns/bundle/bundle.yaml ubuntu@<IP_JUJU>
```

Deploy the bundle and check the result:

```bash
juju deploy ./bundle.yaml --map-machines=existing,0=0,1=1,2=2,3=3
#juju deploy ./bundle.yaml --map-machines=existing,0=30,1=31,2=32,3=33
juju status
```

# Interacting with the Kubernetes cluster

In JUJU machine:

```bash
mkdir -p ~/.kube
juju scp kubernetes-master/0:config ~/.kube/config
sudo snap install kubectl --classic
kubectl cluster-info
```

# To access the K8s dashboard 

In JUJU machine:

```bash
juju config kubernetes-master enable-dashboard-addons=true
```

The dashboard could be accesible running this kubectl command:

```bash
kubectl proxy --address <IP_JUJU>
```

And now a SSH tunnel could be done from the PC to JUJU, and from the web browser access to the dashboard:

<http://localhost:8001/api/v1/namespaces/kube-system/services/https:kubernetes-dashboard:/proxy/>

# Install and init Helm

## Install

```bash
sudo snap install helm --classic
```

## Init

```bash
# Optional. Create account for tiller
# kubectl --namespace kube-system create serviceaccount tiller
# kubectl create clusterrolebinding tiller-cluster-rule --clusterrole=cluster-admin --serviceaccount=kube-system:tiller
helm init
# helm init --service-account tiller --wait
# kubectl --namespace kube-system patch deploy tiller-deploy -p '{"spec":{"template":{"spec":{"serviceAccount":"tiller"}}}}'
kubectl get all --all-namespaces
```
